import picologging as log
from scapy.all import Packet
from scapy.layers.inet import IP, TCP
from scapy.layers.inet6 import IPv6
from yacryptopan import CryptoPAn

# Global cryptopan initialization
cryptopan: CryptoPAn | None = None


def check_checksum(packet: Packet):
    """
    This function checks if the checksum of a packet is valid.

    :param packet: Scapy Packet to be processed.
    :return: None
    """
    if packet.haslayer(IP):
        original_checksum = packet[IP].chksum
        del packet[IP].chksum
        new_checksum = packet[IP].chksum
        if original_checksum != new_checksum:
            log.error(f"IP Checksum Invalid for package {packet.summary()}")

    if packet.haslayer(IPv6):
        original_checksum = packet[IPv6].chksum
        del packet[IPv6].chksum
        new_checksum = packet[IPv6].chksum
        if original_checksum != new_checksum:
            log.error(f"IPv6 Checksum Invalid for package {packet.summary()}")

    if packet.haslayer(TCP):
        original_checksum = packet[TCP].chksum
        del packet[TCP].chksum
        new_checksum = packet[TCP].chksum
        if original_checksum != new_checksum:
            log.error(f"TCP Checksum Invalid for package {packet.summary()}")


def configure_cryptopan(key: bytes) -> None:
    """
    This function configures the CryptoPAN algorithm with the provided key.

    :param key: Key to be used for the CryptoPAN algorithm.
    :return: None
    """
    global cryptopan
    cryptopan = CryptoPAn(key)


def initial_logging_config() -> None:
    """
    This function configures the log for the original file using the picologging module.

    :param original_filename: Original file name.
    :return: None
    """
    log.basicConfig(
        filemode="w",
        level=log.INFO,
        format="%(asctime)s - %(levelname)s - %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
    )


def change_log_file(outDir: str, outName: str) -> None:
    """
    This function changes the logging file for the original file using the picologging module.

    :param original_filename: Original file name.
    :return: None
    """
    logger = log.getLogger()

    for handler in logger.handlers:
        logger.removeHandler(handler)
        handler.close()

    handler = log.FileHandler(f"{outDir}/{outName}.log.txt", mode="w")
    formatter = log.Formatter(
        "%(asctime)s - %(levelname)s - %(message)s", datefmt="%Y-%m-%d %H:%M:%S"
    )

    handler.setLevel(log.INFO)
    handler.setFormatter(formatter)
    logger.addHandler(handler)


def get_cryptopan() -> CryptoPAn:
    """
    This function returns the global CryptoPAn instance.

    :return: CryptoPAn instance.
    """
    if cryptopan is None:
        raise Exception("CryptoPAn not configured")
    return cryptopan


def safe_setattr(obj, attr_name, value):
    """
    This function sets an attribute of an object if it exists, otherwise it does nothing.

    :param obj: The object to set the attribute on.
    :param attr_name: The name of the attribute to set.
    :param value: The value to set the attribute to.
    """
    if hasattr(obj, attr_name):
        setattr(obj, attr_name, value)



import picologging as log
from scapy.all import Packet
from scapy.contrib.igmp import IGMP
from scapy.contrib.igmpv3 import IGMPv3
from scapy.layers.inet import IP, TCP, UDP


def recalculate(packet: Packet, index: int) -> Packet:
    """
    This function recalculates the checksums of the packets.

    :param packet: Scapy Packet to be processed.
    :param index: Index of the packet in the packet list (for logging purposes).
    :return: Anonymized packet.
    """
    try:
        if packet.haslayer(IP):
            del packet[IP].chksum

        if packet.haslayer(TCP):
            del packet[TCP].chksum

        if packet.haslayer(IGMP):
            del packet[IGMP].chksum

        if packet.haslayer(IGMPv3):
            del packet[IGMPv3].chksum

        if packet.haslayer(UDP):
            del packet[UDP].chksum

        return packet
    except Exception as e:
        log.error(f"Error while recalculating checksum for packet {index}: {e}")
        return packet



import picologging as log
from scapy.all import Packet
from scapy.layers.inet import IP
from scapy.layers.inet6 import IPv6
from scapy.layers.l2 import ARP

from algorithms import prefix_preserving_cryptopan as anonymize_ip


def anon_ip_address(packet: Packet) -> Packet | None:
    """
    Anonimize the IP address of a packet.
    This function also checks for ARP packets and anonymizes the source and destination IP addresses to avoid
    leaking original information on ARP packets.

    The anonymization process utilizes the CryptoPAn algorithm to generate a randomized IP address in a prefix-preserving manner.

    :param packet: Scapy Packet to be processed.
    :return: Anonymized packet.
    """
    try:
        if packet.haslayer(IP):
            packet[IP].src = anonymize_ip(packet[IP].src)
            packet[IP].dst = anonymize_ip(packet[IP].dst)
        if packet.haslayer(IPv6):
            packet[IPv6].src = anonymize_ip(packet[IPv6].src)
            packet[IPv6].dst = anonymize_ip(packet[IPv6].dst)

        if packet.haslayer(ARP):
            packet[ARP].psrc = anonymize_ip(packet[ARP].psrc)
            packet[ARP].pdst = anonymize_ip(packet[ARP].pdst)
        return packet
    except Exception as e:
        log.error(f"Error anonymizing IP addresses: {e}")



# steps/__init__.py

from .check_tcp import check_tcpinfo
from .config import (
    change_log_file,
    check_checksum,
    configure_cryptopan,
    get_cryptopan,
    initial_logging_config,
    safe_setattr,
)
from .validate import validate_anonymization

__all__ = [
    "check_checksum",
    "configure_cryptopan",
    "initial_logging_config",
    "change_log_file",
    "get_cryptopan",
    "safe_setattr",
    "validate_anonymization",
    "check_tcpinfo",
]



import argparse
import glob
import os
import signal
import sys

from anonymizer import PcapAnonymizer
from utils import (
    change_log_file,
    check_tcpinfo,
    initial_logging_config,
    validate_anonymization,
)


def signal_handler(sig, frame):
    print("\nInterrupt received. Exiting...")
    sys.exit(0)


def main():
    # Setup signal handler for SIGINT
    signal.signal(signal.SIGINT, signal_handler)

    initial_logging_config()

    parser = argparse.ArgumentParser(
        description="PcapBlur is a tool for anonymizing network traffic captured in .pcap files."
    )

    # Create mutually exclusive group
    group = parser.add_mutually_exclusive_group(required=True)

    group.add_argument(
        "path", nargs="?", help="Path to the .pcap file to be anonymized."
    )

    group.add_argument("--batch", help="Specify a folder for batch anonymization.")

    group.add_argument(
        "--version", "-v", help="Show the version of the tool.", action="store_true"
    )

    group.add_argument(
        "--validate",
        nargs=2,
        metavar=("first", "second"),
        help="Validate anonymization by comparing two .pcap files.",
    )

    group.add_argument(
        "--check-tcp",
        nargs=2,
        metavar=("first", "second"),
        help="Check TCP information by comparing two .pcap files.",
    )

    parser.add_argument(
        "--outDir",
        "-o",
        help="Set the output directory for the anonymized .pcap file(s).",
        default="output",
    )
    parser.add_argument(
        "--outName",
        "-n",
        help="Set the filename of the anonymized .pcap file. (OPTIONAL, works with path only)",
    )

    args = parser.parse_args()

    if args.version:
        print("pcap_blur version 1.0.0")
        return

    if args.check_tcp:
        print("Checking TCP information. This can take a while...")
        # Handling TCP information check for two pcap files
        first_file, second_file = args.check_tcp
        if not os.path.exists(first_file):
            print(f"Error: Original file {first_file} does not exist.")
            return

        if not os.path.exists(second_file):
            print(f"Error: Anonymized file {second_file} does not exist.")
            return

        check_tcpinfo(first_file, second_file)
        return

    if args.validate:
        # Handling validation of two pcap files
        first_file, second_file = args.validate
        if not os.path.exists(first_file):
            print(f"Error: Original file {first_file} does not exist.")
            return

        if not os.path.exists(second_file):
            print(f"Error: Anonymized file {second_file} does not exist.")
            return

        validate_anonymization(first_file, second_file)
        return

    if args.batch:
        # Handling batch anonymization for a folder
        if args.outName:
            parser.error("--outName cannot be used with --batch")

        print(f"Batch anonymization for folder: {args.batch}")

        out_folder = os.path.join(args.batch, args.outDir)
        out_folder_logs = os.path.join(out_folder, "logs")

        if not os.path.exists(out_folder):
            os.makedirs(out_folder)

        if not os.path.exists(out_folder_logs):
            os.makedirs(out_folder_logs)

        # Search for all .pcap or .pcapng files in the folder

        pcap_files = glob.glob(os.path.join(args.batch, "*.pcap")) + glob.glob(
            os.path.join(args.batch, "*.pcapng")
        )

        for pcap_file in pcap_files:
            out_name = os.path.basename(pcap_file).replace(".pcap", ".anon.pcap")
            change_log_file(out_folder_logs, out_name)
            pcap_anonymizer = PcapAnonymizer(pcap_file, out_folder, out_name)
            pcap_anonymizer.anonymize_file()

    else:
        # Handling single file anonymization
        if args.outName is None:
            args.outName = os.path.basename(args.path).replace(
                ".pcap", "_anonymized.pcap"
            )

        if not os.path.exists(args.outDir):
            os.makedirs(args.outDir)

        if not os.path.exists(args.path):
            print(f"Error: The file {args.path} does not exist.")
            return

        file_name = os.path.basename(args.path)

        change_log_file(args.outDir, file_name)
        pcap_anonymizer = PcapAnonymizer(args.path, args.outDir, args.outName)
        pcap_anonymizer.anonymize_file()


if __name__ == "__main__":
    main()



import random
from typing import Dict

from scapy.all import Packet
from scapy.layers.inet import IP, TCP, UDP

# Original port number to permuted port number mapping
port_map: Dict[int, int] = {}


def create_port_permutation(port: int) -> int:
    """
    This function creates a permutation of a port, ensuring that each original port is mapped to a unique permuted port.

    :param port: Original port number.
    :return: Permuted port number.
    """

    # Check if the port has a permutation
    if port in port_map:
        return port_map[port]
    else:
        # Generate a random permutation for the port
        permuted_port = random.randint(0, 65535)
        while permuted_port in port_map.values():
            # If the permutation already exists, generate another
            permuted_port = random.randint(0, 65535)

        # Add the permutation to the dictionary
        port_map[port] = permuted_port

    return permuted_port


def anon_port_numbers(packet: Packet) -> Packet:
    """
    Processes a packet Scapy, applying port permutation if it is a TCP or UDP packet.

    :param packet: Scapy Packet to be processed.
    :return: Anonymized packet.
    """

    if packet.haslayer(IP):
        if packet.haslayer(TCP):
            # Aplicar permutação de porta
            packet[TCP].sport = create_port_permutation(packet[TCP].sport)
            packet[TCP].dport = create_port_permutation(packet[TCP].dport)
        if packet.haslayer("UDP"):
            # Aplicar permutação de porta
            packet[UDP].sport = create_port_permutation(packet[UDP].sport)
            packet[UDP].dport = create_port_permutation(packet[UDP].dport)
    return packet



from scapy.all import Packet
from scapy.layers.l2 import ARP, Ether

from algorithms import generate_permutation_mac


def check_group_address(mac: str) -> bool:
    """
    Check if a MAC address is a group address.

    :param mac: MAC address as a string.
    :return: True if it's a group address, False otherwise.
    """
    # Get the first octet and convert it to integer
    first_octet = int(mac[:2], 16)
    # Check the least significant bit
    is_group_address = (first_octet & 0x01) == 0x01

    return is_group_address


def anonimize_mac(mac_addr: str, is_group_address=False) -> str:
    """
    Auxiliary function to anonymize a MAC address.

    :param mac_addr: MAC address as a string.
    :param is_group_address: True if the address is a group address, False otherwise.
    :return: Anonymized MAC address as a string.
    """

    manufacturer_addr, device_addr = mac_addr[:8], mac_addr[9:]

    mac_result = (
        generate_permutation_mac(
            manufacturer_addr, is_group_address, is_device_address=False
        )
        + ":"
        + generate_permutation_mac(
            device_addr, is_group_address, is_device_address=True
        )
    )

    return mac_result


def anon_mac_address(packet: Packet) -> Packet:
    """
    Anonimize the MAC address of a packet.
    This funtion also checks for ARP packets and anonymizes the source and destination MAC addresses to avoid
    leaking original information on ARP packets.

    :param packet: Scapy Packet to be processed.
    :return: Anonymized packet.
    """
    if packet.haslayer(Ether):
        mac_src = packet[Ether].src
        mac_dst = packet[Ether].dst

        is_group_address_src = check_group_address(mac_src)
        is_group_address_dst = check_group_address(mac_dst)

        mac_src_anon = anonimize_mac(mac_src, is_group_address_src)
        mac_dst_anon = anonimize_mac(mac_dst, is_group_address_dst)

        # Definir endereços MAC anonimizados no pacote
        packet[Ether].src = mac_src_anon
        packet[Ether].dst = mac_dst_anon

        if packet.haslayer(ARP):
            packet[ARP].hwsrc = mac_src_anon
            packet[ARP].hwdst = mac_dst_anon
    return packet



import random
from typing import Dict

port_map: Dict[int, int] = {}

manufacturer_map: Dict[str, str] = {}
device_map: Dict[str, str] = {}


def generate_permutation_mac(
    mac_part: str, is_group_address: bool = False, is_device_address: bool = False
) -> str:
    """
    Generates a permutation for a part of the MAC address, ensuring it retains group or individual characteristics.

    :param mac_part: Part of the MAC address to be permuted.
    :param mac_map: Dictionary for permutation mapping.
    :param is_group_address: Flag to ensure the resulting MAC is a group address.
    :return: Permuted part of the MAC address.
    """

    mac_map = device_map if is_device_address else manufacturer_map

    if mac_part in mac_map:
        return mac_map[mac_part]
    else:
        # Generate random permutation
        permuted_part = ":".join(["%02x" % random.randint(0, 255) for _ in range(3)])
        while permuted_part in mac_map.values():
            permuted_part = ":".join(
                ["%02x" % random.randint(0, 255) for _ in range(3)]
            )

        # Adjust the first byte to respect the group address property
        bytes = permuted_part.split(":")
        first_byte = int(bytes[0], 16)
        if is_group_address:
            first_byte |= 0x01  # Set LSB to 1
        else:
            first_byte &= 0xFE  # Set LSB to 0
        bytes[0] = "%02x" % first_byte
        permuted_part = ":".join(bytes)

        mac_map[mac_part] = permuted_part
        return permuted_part


def generate_permutation_port(port: int) -> int:
    """
    This function creates a permutation of a port, ensuring that each original port is mapped to a unique permuted port.

    :param port: Original port number.
    :return: Permuted port number.
    """

    # Check if the port has a permutation
    if port in port_map:
        return port_map[port]
    else:
        # Generate a random permutation for the port
        permuted_port = random.randint(0, 65535)
        while permuted_port in port_map.values():
            # If the permutation already exists, generate another
            permuted_port = random.randint(0, 65535)

        # Add the permutation to the dictionary
        port_map[port] = permuted_port

    return permuted_port



from datetime import datetime, timezone

import tzlocal
from scapy.utils import EDecimal


def precision_degradation(timestamp: EDecimal) -> EDecimal:
    """
    Converts the timestamp EDecimal to a float and then applies the precision degradation algorithm.

    :param timestamp: EDecimal timestamp to be converted and degraded.
    :return: Degraded timestamp.
    """
    timestamp_float = float(timestamp)

    local_tz = tzlocal.get_localzone()

    timestamp_datetime = datetime.fromtimestamp(timestamp_float, tz=timezone.utc)

    timestamp_datetime_local = timestamp_datetime.astimezone(local_tz)

    degraded_datetime_local = timestamp_datetime_local.replace(microsecond=0, second=0)

    degraded_datetime_utc = degraded_datetime_local.astimezone(timezone.utc)

    degraded_edecimal = EDecimal(degraded_datetime_utc.timestamp())
    return degraded_edecimal



import picologging as log
from scapy.all import Packet
from scapy.layers.inet import ICMP
from scapy.layers.inet6 import IPv6
from yacryptopan import CryptoPAn

from utils import get_cryptopan


def anon_icmp_v4(packet: Packet, index: int, cp: CryptoPAn) -> Packet:
    if packet.haslayer(ICMP):
        log.debug(f"Anonymizing ICMP packet at index {index}.")
        for field in ["gw", "addr_mask"]:
            if getattr(packet[ICMP], field, None) is not None:
                setattr(packet[ICMP], field, cp.anonymize(getattr(packet[ICMP], field)))

        for field in ["ts_ori", "ts_rx", "ts_tx"]:
            if getattr(packet[ICMP], field, None) is not None:
                setattr(packet[ICMP], field, 0)

        del packet[ICMP].chksum
    return packet


def anon_icmp_v6(packet: Packet, index: int, cp: CryptoPAn) -> Packet:
    if packet.haslayer(IPv6) and packet[IPv6].nh == 58:
        log.debug(f"Handling ICMPv6 packet at index {index}.")
        # Assuming specific anonymization tasks for ICMPv6 could be defined here
        packet.load = b""  # Consider more granular handling than wiping payload
    return packet


def anon_icmp(packet: Packet, index: int) -> Packet:
    cp = get_cryptopan()
    if cp is None:
        raise Exception("CryptoPAn not configured")

    packet = anon_icmp_v4(packet, index, cp)
    packet = anon_icmp_v6(packet, index, cp)
    return packet



from scapy.all import Packet, Padding, Raw

from algorithms import black_marker


def anon_app_data(packet: Packet) -> Packet:
    """
    This function anonymizes the application data of a packet.
    It replaces the payload of the packet with a black marker of bytes of zeroes.

    :param packet: Scapy Packet to be processed.
    :return: Anonymized packet.
    """

    # Get the last layer of the packet
    last_layer = packet.getlayer(packet.layers()[-1])

    # Check if the last layer is a Raw layer
    if isinstance(last_layer, Raw):
        # Replace the payload of the Raw layer with a black marker of bytes of zeroes
        payload_length = len(last_layer.load)
        last_layer.load = black_marker(payload_length)

    # Check if the last layer is a Padding layer
    elif isinstance(last_layer, Padding):
        # Replace the payload of the Padding layer with a black marker of bytes of zeroes
        payload_length = len(last_layer.load)
        last_layer.load = black_marker(payload_length)

    return packet



import picologging as log
from scapy.all import Packet, PcapReader
from scapy.layers.inet import IP, TCP, UDP
from scapy.layers.inet6 import IPv6
from scapy.layers.l2 import Ether

log.basicConfig(
    filename="validation_log.txt",
    filemode="w",
    level=log.DEBUG,
    format="%(asctime)s - %(levelname)s - %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
)

hasError = False


def check_values(orig_pkt: Packet, anon_pkt: Packet, index: int):
    global hasError

    # Check MAC addresses
    if Ether in orig_pkt and Ether in anon_pkt:
        if (
            orig_pkt[Ether].src == anon_pkt[Ether].src
            or orig_pkt[Ether].dst == anon_pkt[Ether].dst
        ):
            log.error(
                f"Original MAC address found: {orig_pkt[Ether].src} - {orig_pkt[Ether].dst} on packet #{index}"
            )
            hasError = True

    # Check IPv4 addresses
    if IP in orig_pkt and IP in anon_pkt:
        if orig_pkt[IP].src == anon_pkt[IP].src or orig_pkt[IP].dst == anon_pkt[IP].dst:
            log.error(
                f"Original IPV4 address found: {orig_pkt[IP].src} - {orig_pkt[IP].dst} on packet #{index}"
            )
            hasError = True

    # Check IPv6 addresses
    if IPv6 in orig_pkt and IPv6 in anon_pkt:
        if (
            orig_pkt[IPv6].src == anon_pkt[IPv6].src
            or orig_pkt[IPv6].dst == anon_pkt[IPv6].dst
        ):
            log.error(
                f"Original IPV6 address found: {orig_pkt[IPv6].src} - {orig_pkt[IPv6].dst} on packet #{index}"
            )
            hasError = True

    # Check port numbers TCP
    if TCP in orig_pkt and TCP in anon_pkt:
        if (
            orig_pkt[TCP].sport == anon_pkt[TCP].sport
            or orig_pkt[TCP].dport == anon_pkt[TCP].dport
        ):
            log.error(
                f"Original port number found: {orig_pkt[TCP].sport} - {orig_pkt[TCP].dport} on packet #{index}"
            )
            hasError = True

    # Check port numbers UDP
    if UDP in orig_pkt and UDP in anon_pkt:
        if (
            orig_pkt[UDP].sport == anon_pkt[UDP].sport
            or orig_pkt[UDP].dport == anon_pkt[UDP].dport
        ):
            log.error(
                f"Original port number found: {orig_pkt[UDP].sport} - {orig_pkt[UDP].dport} on packet #{index}"
            )
            hasError = True


def validate_anonymization(original_pcap_path: str, anonymized_pcap_path: str):
    try:
        print("Validating anonymization. This might take a while...")
        with PcapReader(original_pcap_path) as original_reader, PcapReader(
            anonymized_pcap_path
        ) as anonymized_reader:
            for index, (original_pkt, anon_pkt) in enumerate(
                zip(original_reader, anonymized_reader)
            ):
                check_values(original_pkt, anon_pkt, index)

        if hasError:
            log.error(
                "Anonymization failed. Some original information was found in the anonymized packets."
            )
        else:
            log.info(
                "Anonymization successful. No original information was found in the anonymized packets."
            )

        print("Anonymization validation finished.")
        print("Check ./validation_log.txt for more information.")
    except Exception as e:
        print(f"Anonymization validation failed: {e}")



from datetime import datetime, timezone

import tzlocal
from scapy.all import Packet
from scapy.layers.inet import IP, TCP, IPOption_Timestamp
from scapy.utils import EDecimal


def degrade_edecimal_timestamp(timestamp: EDecimal) -> EDecimal:
    """
    Converts the timestamp EDecimal to a float and then degrades it to the nearest millisecond.

    :param timestamp: EDecimal timestamp to be converted and degraded.
    :return: Degraded timestamp.
    """
    timestamp_float = float(timestamp)

    local_tz = tzlocal.get_localzone()

    timestamp_datetime = datetime.fromtimestamp(timestamp_float, tz=timezone.utc)

    timestamp_datetime_local = timestamp_datetime.astimezone(local_tz)

    degraded_datetime_local = timestamp_datetime_local.replace(microsecond=0)

    degraded_datetime_utc = degraded_datetime_local.astimezone(timezone.utc)

    degraded_edecimal = EDecimal(degraded_datetime_utc.timestamp())
    return degraded_edecimal


def degrade_integer_timestamp(timestamp: int) -> int:
    """
    Degrades a TCP timestamp to the nearest millisecond.

    :param timestamp: The timestamp to be degraded.
    :return: The degraded timestamp.
    """
    return int(timestamp / 1000) * 1000


def anon_timestamps(packet: Packet) -> Packet:
    """
    Anonymizes the timestamps of a packet using the precision degradation function.
    This function will use the Precision Degradation algorithm to degrade the timestamps to the nearest millisecond.
    This function will degrade Timestamps present on the Packet Metadata as well as TCP Timestamp Options and IP (and IPv6) Timestamp Options.

    :param packet: Scapy Packet to be processed.
    :return: Anonymized packet.
    """

    # Degrade the timestamp in the packet metadata
    degraded_packet_ts = degrade_edecimal_timestamp(packet.time)
    packet.time = degraded_packet_ts

    # Degrade the timestamps in the TCP options (if any)
    if packet.haslayer(TCP):
        # Get TCP timestamp option if it exists
        tcp_options = packet.getlayer(TCP).options

        # Options is a list of tuples
        # Find the option where the first element is "Timestamp"
        original_timestamp_option = next(
            (opt for opt in tcp_options if opt[0] == "Timestamp"), None
        )

        if original_timestamp_option is not None:
            # The timestamp option has the following format:
            # ("Timestamp", (TSval, Tsecr))
            timestamps = original_timestamp_option[1]

            # Iterate over the timestamps tuple
            degraded_timestamps = ()
            for ts in timestamps:
                degraded_ts = degrade_integer_timestamp(ts)
                degraded_timestamps += (degraded_ts,)

            # Set the degraded timestamps in the TCP option
            new_timestamp_option = ("Timestamp", degraded_timestamps)
            tcp_options.remove(original_timestamp_option)
            tcp_options.insert(0, new_timestamp_option)

        packet.getlayer(TCP).options = tcp_options

    # Degrade the timestamps in the IPv4 options (if any)
    if packet.haslayer(IP):
        # Get IP timestamp option if it exists
        ip_opt = packet.getlayer(IP).options

        print("original options", ip_opt)
        # Iterate the list of options
        for i, opt in enumerate(ip_opt):
            # Check if the option is of class IPOption_Timestamp
            if isinstance(opt, IPOption_Timestamp):
                # Degrade the timestamp
                degraded_timestamp = degrade_integer_timestamp(opt.timestamp)
                opt.timestamp = degraded_timestamp

                # Set the new timestamp in the IPOption_Timestamp object
                packet.getlayer(IP).options[i] = opt

    return packet



def black_marker(length: int) -> bytes:
    """
    This function returns a dull black marker of bytes of zeroes of the specified length.

    :param lenght: Length of the black marker.
    :return: Black marker of bytes of zeroes.
    """
    return b"\x00" * length



# steps/__init__.py

from .app_data import anon_app_data
from .checksum import recalculate
from .icmp import anon_icmp
from .ip_address import anon_ip_address
from .mac_address import anon_mac_address
from .port_numbers import anon_port_numbers
from .timestamps import anon_timestamps

__all__ = [
    "anon_app_data",
    "recalculate",
    "anon_timestamps",
    "anon_port_numbers",
    "anon_mac_address",
    "anon_ip_address",
    "anon_icmp",
]



# algorithms/__init__.py

from .black_marker import black_marker
from .permutation import generate_permutation_mac, generate_permutation_port
from .precision_degradation import precision_degradation
from .prefix_preserving_cryptopan import prefix_preserving_cryptopan

__all__ = [
    "black_marker",
    "prefix_preserving_cryptopan",
    "generate_permutation_port",
    "generate_permutation_mac",
    "precision_degradation",
]



from scapy.all import Packet, rdpcap
from scapy.layers.inet import TCP


class TcpInfo:
    def __init__(self):
        self.packets_info = []

    def analyze_packet(self, packet: Packet):
        if packet.haslayer(TCP):
            tcp_layer = packet.getlayer(TCP)

            # Packet information
            packet_info = {
                "flags": tcp_layer.flags,
                "seq": tcp_layer.seq,
                "ack": tcp_layer.ack,
                "history": [],
            }

            if tcp_layer.flags & 0x02:  # SYN
                packet_info["history"].append("SYN")
            if tcp_layer.flags & 0x12:  # SYN+ACK
                packet_info["history"].append("SYN+ACK")
            if tcp_layer.flags & 0x10:  # ACK
                packet_info["history"].append("ACK")
            if tcp_layer.flags & 0x01:  # FIN
                packet_info["history"].append("FIN")

            self.packets_info.append(packet_info)

    def read_pcap(self, filename):
        packets = rdpcap(filename)
        for packet in packets:
            self.analyze_packet(packet)

    def compare_packets(self, other):
        len_self = len(self.packets_info)
        len_other = len(other.packets_info)

        if len_self != len_other:
            print(
                f"Packet count mismatch: {len_self} packets in file 1, {len_other} packets in file 2."
            )

        for i in range(len_self):
            packet1 = self.packets_info[i]
            packet2 = other.packets_info[i]

            if packet1 != packet2:
                print(f"Difference in packet {i + 1}:")
                for key in packet1:
                    if packet1[key] != packet2[key]:
                        print(
                            f"  {key}: {packet1[key]} (file 1) != {packet2[key]} (file 2)"
                        )
                print(packet1)
                print(packet2)

        if len_self != len_other:
            print(
                f"Packet count mismatch: {len_self} packets in file 1, {len_other} packets in file 2."
            )
            if len_self > len_other:
                print(f"Extra packets in file 1 starting from index {len_other + 1}:")
                for i in range(len_other, len_self):
                    print(self.packets_info[i])
            else:
                print(f"Extra packets in file 2 starting from index {len_self + 1}:")
                for i in range(len_self, len_other):
                    print(other.packets_info[i])


def check_tcpinfo(file1, file2):
    tcpinfo1 = TcpInfo()
    tcpinfo2 = TcpInfo()

    tcpinfo1.read_pcap(file1)
    tcpinfo2.read_pcap(file2)

    tcpinfo1.compare_packets(tcpinfo2)

    print("TCP information check finished.")



from setuptools import find_packages, setup

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

with open("README.md") as f:
    long_description = f.read()

setup(
    name="pcap_blur",
    version="2.0.0",
    description="Pcap Blur is a command line tool that anonymizes network traffic.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rafaelsilva81/pcap-blur",
    author="Rafael Galdino da Silva",
    author_email="rafaelgaldinosilva81@gmail.com",
    license="MIT",
    packages=find_packages(),
    py_modules=["pcap_blur"],  # Since pcap_blur.py is in the root
    entry_points={
        "console_scripts": [
            "pcap-blur=pcap_blur:main",
            "pcap_blur=pcap_blur:main",
            "pcapblur=pcap_blur:main",
        ],
    },
    install_requires=requirements,
    python_requires=">=3.10",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Topic :: Security",
        "Topic :: Security :: Cryptography",
        "Topic :: System :: Networking",
        "Topic :: Utilities",
    ],
)



import os

from yacryptopan import CryptoPAn

cryptopan = CryptoPAn(os.urandom(32))


def configure_cryptopan(key: bytes) -> None:
    """
    This function configures the CryptoPAn algorithm with the provided key.

    :param key: Key to be used for the CryptoPAn algorithm.
    :return: None
    """
    global cryptopan
    cryptopan = CryptoPAn(key)


def prefix_preserving_cryptopan(address: str) -> str:
    """
    This function anonymizes an IP address using the prefix-preserving cryptopan algorithm.

    :param address: IP address to be anonymized.
    :return: Anonymized IP address.
    """

    return cryptopan.anonymize(address)
